@@ -2,6 +2,7 @@
 
 import decimal
 import enum
+import functools
 import hashlib
 import string
 
@@ -98,23 +99,13 @@ class UEID:
         :raises TypeError: If the value's type is not encodable.
         """
         try:
-            type_, encoder = {
-                type(None): (_Type.nil, cls._encode_nil),
-                bool: (_Type.bool, cls._encode_bool),
-                int: (_Type.number, cls._encode_number),
-                float: (_Type.number, cls._encode_number),
-                str: (_Type.unicode, cls._encode_unicode),
-                bytes: (_Type.bytes, cls._encode_bytes),
-                list: (_Type.array, cls._encode_array),
-                dict: (_Type.mapping, cls._encode_mapping),
-                UEID: (_Type.unicode,
-                       lambda ueid: cls._encode_unicode(str(ueid)))
-            }[type(value)]
+            type_, encoder = _ENCODERS[type(value)]
         except KeyError:
             raise TypeError(
                 'Cannot encode value of type {!r}'.format(type(value)))
         return bytes([type_]) + encoder(value)
 
+
     @staticmethod
     def _encode_nil(nil):
         """Encode a null value."""
@@ -129,6 +120,7 @@ class UEID:
         return b'\x01' if bool_ else b'\x00'
 
     @classmethod
+    @functools.lru_cache(maxsize=16384)
     def _encode_number(cls, number):
         """Encode a numeric value.
 
@@ -140,6 +132,7 @@ class UEID:
         return cls._encode_unicode(format(decimal_, '+.64f'))
 
     @classmethod
+    @functools.lru_cache(maxsize=16384)
     def _encode_unicode(cls, unicode_):
         """Encode a Unicode string.
 
@@ -194,6 +187,7 @@ class UEID:
         return cls._encode_mapping(dict(enumerate(array)))
 
     @staticmethod
+    @functools.lru_cache(maxsize=16384)
     def _encode_integer(integer):
         """Encode integer as bytes.
 
@@ -242,3 +236,19 @@ class UEID:
             raise UEIDError(str(exc)) from exc
         ueid_hex = ''.join(hex(byte)[2:].zfill(2) for byte in ueid_bytes)
         return cls(ueid_hex)
+
+
+#: Type-to-codec mapping for UEID value encoding.
+#: See :meth:`UEID._encode_value`.
+_ENCODERS = {
+    type(None): (_Type.nil, UEID._encode_nil),
+    bool: (_Type.bool, UEID._encode_bool),
+    int: (_Type.number, UEID._encode_number),
+    float: (_Type.number, UEID._encode_number),
+    str: (_Type.unicode, UEID._encode_unicode),
+    bytes: (_Type.bytes, UEID._encode_bytes),
+    list: (_Type.array, UEID._encode_array),
+    dict: (_Type.mapping, UEID._encode_mapping),
+    UEID: (_Type.unicode,
+          lambda ueid: UEID._encode_unicode(str(ueid))),
+}
